﻿using System;
using System.Drawing;
using System.IO;
using System.Threading;
using System.Windows.Forms;
using TopoR_PCB_Classes;

namespace TopoRAdd
{
    public partial class AddLibraryForm : Form
    {

        public TopoR_PCB_File donorFST = new TopoR_PCB_File();
        private string _fileName;

        public AddLibraryForm()
        {
            InitializeComponent();
        }
        private void asyncload()
        {
            donorFST = XMLFile.LoadFST<TopoR_PCB_File>(_fileName);
            BeginInvoke(_closeProgress);
        }
        private delegate void TaskDone();
        private TaskDone _closeProgress;
        private void btFileOpen_Click(object sender, EventArgs e)
        {
            if ((Owner as MainWindow).openFD.ShowDialog() == DialogResult.OK)
            {
                _fileName = (Owner as MainWindow).openFD.FileName;
                using (ProgressForm p = new ProgressForm())
                {
                    _closeProgress = new TaskDone(p.Close);
                    Thread task = new Thread(asyncload);
                    task.Start();
                    p.ShowDialog();
                }
                groupBox1.Text = "Библиотека компонентов файла " + Path.GetFileNameWithoutExtension(_fileName);
                lbDonorLibrary.Items.Clear();
                if (donorFST?._LocalLibrary._Packages?.Count > 0)
                {
                    lbDonorLibrary.Items.AddRange(donorFST._LocalLibrary._Packages.ToArray());
                }
            }
        }
        private void asyncAddLibrary()
        {
            foreach (LocalLibrary.Package lp in lbLibrary.Items)
            {
                if (lp.Tag == null)
                {
                    (Owner as MainWindow).docFST.AddPackageFromLibrary(donorFST._LocalLibrary, donorFST._Header._Units._dist, lp);
                }
            }
            BeginInvoke(_closeProgress);
        }
        private void btOk_Click(object sender, EventArgs e)
        {
            using (ProgressForm p = new ProgressForm())
            {
                _closeProgress = new TaskDone(p.Close);
                Thread task = new Thread(asyncAddLibrary);
                task.Start();
                p.ShowDialog();
            }
            DialogResult = DialogResult.OK;
        }

        private void lbLibrary_DrawItem(object sender, DrawItemEventArgs e)
        {
            if (e.Index < 0)
            {
                return;
            }
            e.DrawBackground();
            using (Brush b = new SolidBrush(((e.State & DrawItemState.Selected) == DrawItemState.Selected) ? Color.White : lbLibrary.ForeColor),
                         bu = new SolidBrush(((e.State & DrawItemState.Selected) == DrawItemState.Selected) ? Color.White : Color.DarkBlue))
            {
                e.Graphics.DrawString(((sender as ListBox).Items[e.Index] as LocalLibrary.Package)._ComponentRef._ReferenceName, lbLibrary.Font, b, e.Bounds);
                if (((sender as ListBox) == lbLibrary && chbFootPrint.Checked) || ((sender as ListBox) == lbDonorLibrary && chbSourceFootPrint.Checked))
                {
                    e.Graphics.DrawString(((sender as ListBox).Items[e.Index] as LocalLibrary.Package)._FootprintRef._ReferenceName, lbLibrary.Font, bu, e.Bounds, new StringFormat(StringFormatFlags.DirectionRightToLeft));
                }
            }
        }

        private void lbLibrary_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (lbLibrary.SelectedItem != null)
            {
                lbDonorLibrary.SelectedIndex = -1;
                btAdd.Enabled = false;
                btRemove.Enabled = (((lbLibrary.SelectedItem as LocalLibrary.Package).Tag as int?) ?? 255) != 0;
            }
        }

        private void lbDonorLibrary_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (lbDonorLibrary.SelectedItem != null)
            {
                lbLibrary.SelectedIndex = -1;
                btRemove.Enabled = false;
                btAdd.Enabled = !lbLibrary.Items.Contains(lbDonorLibrary.SelectedItem);
            }
        }

        private void chbFootPrint_CheckedChanged(object sender, EventArgs e)
        {
            lbLibrary.Invalidate();
            lbDonorLibrary.Invalidate();
        }

        private void btAdd_Click(object sender, EventArgs e)
        {
            lbLibrary.Items.Add(lbDonorLibrary.SelectedItem);
            btAdd.Enabled = false;
        }

        private void btRemove_Click(object sender, EventArgs e)
        {
            lbLibrary.Items.Remove(lbLibrary.SelectedItem);
        }
    }
}
