﻿using TopoR_PCB_Classes;
using System;
using System.Collections.Generic;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Windows.Forms;
using System.Threading;
using System.Diagnostics;

namespace TopoRAdd
{
    public partial class MainWindow : Form
    {
        private string _fileName;
        public TopoR_PCB_File docFST = new TopoR_PCB_File();
     
        public MainWindow()
        {
            InitializeComponent();
            nudX.Maximum = nudY.Maximum = nudAngle.Maximum = decimal.MaxValue;
            nudX.Minimum = nudY.Minimum = nudAngle.Minimum = decimal.MinValue;
            XMLFile.EventXmlSer += new EventHandler<EventXmlSerializer>(ErrorMessage);
        }
        private void ErrorMessage(object sender, EventXmlSerializer e)
        {
            MessageBox.Show(e.Message);
        }

        private void MainWindow_Shown(object sender, EventArgs e)
        {
            string[] arguments = Environment.GetCommandLineArgs();
            if (arguments.Length > 1)
            {
                if (Path.GetExtension(arguments[1]) == ".fst")
                {
                    if (File.Exists(arguments[1]))
                    {
                        _open(arguments[1]);
                    }
                }
            }
        }
        #region MainMenu
        #region File
        private void tsmiNew_Click(object sender, EventArgs e)
        {

        }

        private void asyncload()
        {
            docFST = XMLFile.LoadFST<TopoR_PCB_File>(_fileName);
            BeginInvoke(_closeProgress);
        }
        private void asyncsave()
        {
            XMLFile.SaveFST(docFST, _fileName);
            BeginInvoke(_closeProgress);
        }

        private delegate void TaskDone();
        private TaskDone _closeProgress;
        private void _open(string filename)
        {
            _fileName = filename;
            using (ProgressForm p = new ProgressForm())
            {
                _closeProgress = new TaskDone(p.Close);
                Thread task = new Thread(asyncload);                
                task.Start();
                p.ShowDialog();
            }
            if (docFST != null)
            {
                Text = "TopoRAdd - " + _fileName;
                tsbSave.Enabled = tsmiSave.Enabled = btAddLib.Enabled = btAdd.Enabled = true;
                lbVersion.Text = "Версия файла: " + docFST._Header._Version + "   Единицы измерения: " + docFST._Header._Units._dist.ToString();
                lbXU.Text = lbYU.Text = docFST._Header._Units._dist.ToString();
                nudX.DecimalPlaces = nudY.DecimalPlaces = nudAngle.DecimalPlaces = docFST._Header._Units._dist.ToString() == "mil" ? 3 : 4;
                Layers();
                Components();
                LocalLibrary();
            }
            else
            {
                tsbSave.Enabled = tsmiSave.Enabled = btAddLib.Enabled = btAdd.Enabled = false;
            }
        }
        private void tsmiOpen_Click(object sender, EventArgs e)
        {
            if (openFD.ShowDialog() == DialogResult.OK)
            {
                _open(openFD.FileName);
            }
        }

        private void tsmiSave_Click(object sender, EventArgs e)
        {
            saveFD.InitialDirectory = openFD.InitialDirectory;
            saveFD.FileName = Path.GetFileNameWithoutExtension(_fileName) + "_modified.fst";
            if (saveFD.ShowDialog() == DialogResult.OK)
            {
                _fileName = saveFD.FileName;
                using (ProgressForm p = new ProgressForm())
                {
                    _closeProgress = new TaskDone(p.Close);
                    Thread task = new Thread(asyncsave);
                    task.Start();
                    p.ShowDialog();
                }
            }
        }

        private void tsmiExit_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }
        #endregion File
        #region Settings
        private void tsmiSettings_Click(object sender, EventArgs e)
        {
            using (SettingsForm sf = new SettingsForm())
            {
                sf.ShowDialog();
            }
        }
        #endregion Settings
        #region Help
        private void tsmiContent_Click(object sender, EventArgs e)
        {
            string s = Application.StartupPath + @"\TopoRAdd.pdf";
            if (File.Exists(s))
            {
                Process.Start(s);
            }
        }
        private void TsmiAbout_Click(object sender, EventArgs e)
        {
            using (AboutBox a = new AboutBox())
            {
                a.ShowDialog();
            }
        }
        #endregion Help
        #endregion MainMenu
        
        #region TabControl
        #region Layers
        /// <summary>
        /// Заполнение всех ListBoxs наименованиями слоёв
        /// </summary>
        private void Layers()
        {
            layersList.Items.Clear();
            docLayersList.Items.Clear();
            groupsLayersList.Items.Clear();

            if (docFST._Layers?._StackUpLayers?.Count > 0)
            {
                layersList.Items.AddRange(docFST._Layers._StackUpLayers.ToArray());
            }

            if (docFST._Layers?._UnStackLayers?.Count > 0)
            {
                docLayersList.Items.AddRange(docFST._Layers._UnStackLayers.ToArray());
            }

            if (docFST._Groups?._LayerGroups?.Count > 0)
            {
                groupsLayersList.Items.AddRange(docFST._Groups._LayerGroups.ToArray());
            }
        }
        /// <summary>
        /// Выбор режима проверки/исправления
        /// </summary>
        private void RbOnlyCheck_CheckedChanged(object sender, EventArgs e)
        {
            btCheckRepire.Text = rbOnlyCheck.Checked ? "Проверить" : "Проверить и исправить";
            gbRepair.Visible = !rbOnlyCheck.Checked;
        }
        /// <summary>
        /// Проверка и исправление ссылок на слои
        /// </summary>
        private void BtCheckRepire_Click(object sender, EventArgs e)
        {
            int count = 0; //счётчик ошибок
            List<string> errl = new List<string>(); //список ошибочных слоёв
            string newRefLayer = ""; //ссылка на слой, для замены некорректной

            // Формирование строки отчёта об ошибках в разделе
            void report(string cname)
            {
                tbReport.Text += cname + " " + count.ToString() + Environment.NewLine;
                errl.ForEach(delegate (string name)
                {
                    tbReport.Text += "\t" + name + Environment.NewLine;
                });

                count = 0;
                errl.Clear();
            }
            // Проверка корректности ссылки на слой, "BOARD" - вероятно, какой-то системный слой (присутствует в разделе DisplayControl)
            //TODO: что-то жуткое получилось ради доступа к полю _LayerRef у любого объекта. Надо пересмотреть. Может, интерфейс?
            void check<T>(List<T> Li)
            {
                int zl = Li?.Count ?? 0;
                for (; zl > 0; zl--)
                {
                    var v = Li[zl - 1]?.GetType()?.GetField("_LayerRef")?.GetValue(Li[zl - 1]);
                    if (v != null)
                    {
                        if (v is LayerRef)
                        {
                            string rf = (v as LayerRef)._ReferenceName;
                            if ((!docFST._Layers.LayerStackUpContains(v as LayerRef)) && (!docFST._Layers.LayerUnStackContain(v as LayerRef)) && (rf.ToUpper() != "BOARD"))
                            {
                                if (!errl.Contains(rf))
                                {
                                    errl.Add(rf);
                                }
                                count++;
                                if (!rbOnlyCheck.Checked)
                                {
                                    if (cbRemoved.Checked)
                                    {
                                        Li.RemoveAt(zl - 1);
                                    }
                                    else
                                    {
                                        (v as LayerRef)._ReferenceName = newRefLayer;
                                    }
                                }
                            }
                        }
                    }
                }
            }

            // если выбрано исправление заменой на другой слой...
            if (!rbOnlyCheck.Checked)
            {
                if (!cbRemoved.Checked)
                {
                    if (layersList.SelectedIndex < 0)
                    {
                        MessageBox.Show("Необходимо выбрать слой!");
                        return;
                    }
                    newRefLayer = layersList.SelectedItem.ToString();
                }
            }
            tbReport.Text = "Кол-во ошибочных ссылок на слои, по разделам:" + Environment.NewLine + "----------------------------------------------------------------------" + Environment.NewLine;

            #region ComponentsOnBoard
            if (cbComponentsOnBoard.Checked)
            {

                foreach (var a in docFST._ComponentsOnBoard?._Components?.Where(r => r._Attributes != null))
                {
                    foreach (var b in a._Attributes.Where(r => r._Labels != null))
                    {
                        check(b._Labels);
                    }
                }
                report("ComponentsOnBoard\t\t");
            }
            #endregion ComponentsOnBoard
            #region Connectivity
            if (cbConnectivity.Checked)
            {
                check(docFST._Connectivity?._ZippedWires);
                report("Connectivity.ZippedWires\t\t");

                check(docFST._Connectivity?._Wires);
                report("Connectivity.Wires\t\t\t");

                check(docFST._Connectivity?._Coppers);
                report("Connectivity.Coppers\t\t");

                check(docFST._Connectivity?._NonfilledCoppers);
                report("Connectivity.NonfilledCoppers\t\t");
            }
            #endregion Connectivity
            #region Constructive

            if (cbConstructive.Checked)
            {
                check(docFST._Constructive?._MechLayerObjects);
                report("Constructive.MechLayerObjects\t");

                check(docFST._Constructive?._Texts);
                report("Constructive.Texts\t\t\t");

                if (docFST._Constructive?._Keepouts != null)
                {
                    foreach (var a in docFST._Constructive._Keepouts)
                    {
                        check(a._Role?._Trace?._LayersRefs);
                    }
                }
                report("Constructive.Keepouts.Role.Trace\t");
            }
            #endregion Constructive
            #region DialogSettings
            if (cbDialogSettings.Checked)
            {
                check(docFST._DialogSettings?._GerberSettings?._ExportFiles);
                report("DialogSettings.GerberSettings\t\t");

                check(docFST._DialogSettings?._DXFSettings?._ExportLayers);
                report("DialogSettings.DXFSettings\t\t");
            }
            #endregion DialogSettings
            #region DisplayControl
            if (cbDisplayControl.Checked)
            {
                check(docFST._DisplayControl?._LayersVisualOptions);
                report("DisplayControl\t\t\t");
            }
            #endregion DisplayControl
            #region Groups
            if (cbGroups.Checked)
            {
                if (docFST._Groups?._LayerGroups != null)
                {
                    foreach (var a in docFST._Groups._LayerGroups)
                    {
                        check(a._LayerRefs);
                    }
                }
                report("Groups\t\t\t\t");
            }
            #endregion Groups
            #region HiSpeedRules
            if (cbHiSpeedRules.Checked)
            {
                if (docFST._HiSpeedRules?._RulesImpedances?.OfType<HiSpeedRules.Impedance>() != null)
                {
                    foreach (var a in docFST._HiSpeedRules._RulesImpedances.OfType<HiSpeedRules.Impedance>())
                    {
                        check(a._LayerImpedanceRules);
                    }
                }
                if (docFST._HiSpeedRules?._RulesImpedances?.OfType<HiSpeedRules.ImpedanceDiff>() != null)
                {
                    foreach (var a in docFST._HiSpeedRules._RulesImpedances.OfType<HiSpeedRules.ImpedanceDiff>())
                    {
                        check(a._LayerImpedanceDiffRules);
                    }
                }
                report("HiSpeedRules\t\t\t");
            }
            #endregion HiSpeedRules
            #region LocalLibrary
            if (cbLocalLibrary.Checked)
            {
                if (docFST._LocalLibrary?._Padstacks != null)
                {
                    foreach (var a in docFST._LocalLibrary._Padstacks)
                    {
                        check(a._Pads);
                    }
                }
                report("LocalLibrary.Padstacks\t\t");

                if (docFST._LocalLibrary?._Viastacks != null)
                {
                    foreach (var a in docFST._LocalLibrary._Viastacks)
                    {
                        check(a._LayerRange?._LayerRefs);
                        check(a._ViaPads);
                    }
                }
                report("LocalLibrary.Viastacks\t\t");

                if (docFST._LocalLibrary?._Footprints != null)
                {
                    foreach (var a in docFST._LocalLibrary._Footprints)
                    {
                        check(a._Details);
                        check(a._Texts);
                    }
                }
                report("LocalLibrary.Footprints\t\t");
            }
            #endregion LocalLibrary
            #region Rules
            if (cbRules.Checked)
            {
                if (docFST._Rules?._RulesWidthOfWires != null)
                {
                    foreach (var a in docFST._Rules._RulesWidthOfWires)
                    {
                        check(a._LayersRefs);
                    }
                }
                report("Rules.WidthOfWires\t\t");

                if (docFST._Rules?._RulesClearancesNetToNet != null)
                {
                    foreach (var a in docFST._Rules._RulesClearancesNetToNet)
                    {
                        check(a._LayersRefs);
                    }
                }
                report("Rules.ClearancesNetToNet\t\t");

                if (docFST._Rules?._RulesPlaneLayersNets != null)
                {
                    foreach (var a in docFST._Rules._RulesPlaneLayersNets)
                    {
                        check(a._LayersRefs);
                    }
                }
                report("Rules.PlaneLayersNets\t\t");


                if (docFST._Rules?._RulesSignalLayersNets != null)
                {
                    foreach (var a in docFST._Rules._RulesSignalLayersNets)
                    {
                        check(a._LayersRefs);
                    }
                }
                report("Rules.SignalLayersNets\t\t");
            }
        }
        #endregion Rules
        #endregion Layers
        #region ComponentsPage
        #region Components
        /// <summary>
        /// <summary>
        /// Функция заполнения ListView наименованиями компонентов
        /// </summary>
        private void Components()
        {
            lbComponents.Items.Clear();
            if (docFST._ComponentsOnBoard._Components?.Count > 0)
            {
                lbComponents.Items.AddRange(docFST._ComponentsOnBoard._Components.ToArray());
            }
            btArrange.Enabled = lbComponents.Items.Count > 0;
        }
        private void LbComponents_SelectedIndexChanged(object sender, EventArgs e)
        {
            // если выбран только один компонет, то свойствам компонента...
            if (lbComponents.SelectedItems.Count == 1)
            {
                //... установка его значений
                cbSide.SelectedIndex = (lbComponents.SelectedItem as ComponentsOnBoard.CompInstance)._side == side.Top ? 0 : 1;
                nudX.Value = (decimal)(lbComponents.SelectedItem as ComponentsOnBoard.CompInstance)._Org?._x;
                nudY.Value = (decimal)(lbComponents.SelectedItem as ComponentsOnBoard.CompInstance)._Org?._y;
                btLock.ImageIndex = (lbComponents.SelectedItem as ComponentsOnBoard.CompInstance)._fixed == Bool.on ? 0 : 1;
                nudAngle.Value = (decimal)(lbComponents.SelectedItem as ComponentsOnBoard.CompInstance)._angle;

                //нахождение компонента в библиотеке компонентов и выделение в списке
                for (int x = 0; x < lbLibrary.Items.Count; x++)
                {
                    if (lbLibrary.Items[x].ToString() == (lbComponents.SelectedItem as ComponentsOnBoard.CompInstance)._ComponentRef?._ReferenceName)
                    {
                        lbLibrary.SelectedIndex = x;
                        break;
                    }
                }
            }
            else
            {//если компонент выделен не один, то свойствам присваиваются пустые значения.
                cbSide.Text = "";
                nudX.Value = nudY.Value = nudAngle.Value = 0;
                btLock.ImageIndex = 0;
                lbLibrary.ClearSelected();
            }
            //активация/деактивация кнопок "Удалить", "Переименовать"
            btRemove.Enabled = btRename.Enabled = lbComponents.SelectedItems.Count > 0;
        }
        private void chbUniqueID_CheckedChanged(object sender, EventArgs e)
        {
            lbComponents.Invalidate();
        }
        private void lbComponents_DrawItem(object sender, DrawItemEventArgs e)
        {
            if (e.Index < 0)
            {
                return;
            }
            e.DrawBackground();
            using (Brush b = new SolidBrush(((e.State & DrawItemState.Selected) == DrawItemState.Selected) ? Color.White : lbComponents.ForeColor),
                        bu = new SolidBrush(((e.State & DrawItemState.Selected) == DrawItemState.Selected) ? Color.White : Color.DarkBlue)
            )
            {
                e.Graphics.DrawString((lbComponents.Items[e.Index] as ComponentsOnBoard.CompInstance)._name, lbComponents.Font, b, e.Bounds);
                if (chbUniqueID.Checked)
                {
                    e.Graphics.DrawString((lbComponents.Items[e.Index] as ComponentsOnBoard.CompInstance)._uniqueId, lbComponents.Font, bu, e.Bounds, new StringFormat(StringFormatFlags.DirectionRightToLeft));
                }
            }
        }
        #region Add
        private void btAdd_Click(object sender, EventArgs e)
        {
            tbName.Text = "";
            _addCompCheck();
            tlpAddComponent.Visible = true;
            AcceptButton = btAddCompConfirm;
        }
        private void btAddCompConfirm_Click(object sender, EventArgs e)
        {
            if ((sender as Button) == btAddCompConfirm)
            {
                docFST.AddComponent(tbName.Text, lbLibrary.SelectedItem as LocalLibrary.Package);
                Components();
                lbComponents.SelectedIndex = lbComponents.Items.Count - 1;
            }
            AcceptButton = null;
            tlpAddComponent.Visible = false;
        }
        private void _addCompCheck()
        {
            if (lbLibrary.SelectedIndex < 0)
            {
                lbAddCompError.Text = "выберите элемент в Библиотеке =>";
                btAddCompConfirm.Enabled = false;
                return;
            }
            if (docFST._ComponentsOnBoard?.ComponentIndexOf(tbName.Text) != -1)
            {
                tbName.ForeColor = Color.Red;
                lbAddCompError.Text = "такое имя уже есть";
                btAddCompConfirm.Enabled = false;
                return;
            }
            if (tbName.Text.Trim() == "")
            {
                lbAddCompError.Text = "введите уникальное имя компонента.";
                btAddCompConfirm.Enabled = false;
                return;
            }

            tbName.ForeColor = Color.Black;
            lbAddCompError.Text = "";
            btAddCompConfirm.Enabled = true;

        }
        private void lbLibrary_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (tlpAddComponent.Visible)
            {
                _addCompCheck();
            }
        }
        private void tbName_TextChanged(object sender, EventArgs e)
        {
            _addCompCheck();
        }
        #endregion Add
        #region Remove
        private void btRemove_Click(object sender, EventArgs e)
        {
            if (lbComponents.SelectedItems.Count < 0)
            {
                MessageBox.Show("Необходимо выбрать компоненты в \"Списке компонентов на плате\"!");
                return;
            }
            List<string> ls = new List<string>
                    {
                        "Удалить компонент" + (lbComponents.SelectedItems.Count > 1 ? "ы?" : "?")
                    };
            foreach (ComponentsOnBoard.CompInstance a in lbComponents.SelectedItems)
            {
                ls.Add(a._name);
            }
            using (WarningForm wf = new WarningForm("Удаление компонентов", "Удалить", "Отменить", ls))
            {
                if (wf.ShowDialog() == DialogResult.OK)
                {
                    for (int x = lbComponents.SelectedItems.Count; x > 0; x--)
                    {
                        string name = (lbComponents.SelectedItems[x - 1] as ComponentsOnBoard.CompInstance)._name;
                        if (docFST.RemoveComponent(name))
                        {
                            lbComponents.Items.Remove(lbComponents.SelectedItems[x - 1]);
                        }
                    }
                }
            }
        }
        #endregion Remove
        #region Rename
        private void btRename_Click(object sender, EventArgs e)
        {
            if (lbComponents.SelectedIndex < 0)
            {
                MessageBox.Show("Необходимо выбрать компоненты в \"Списке компонентов на плате\"!");
                return;
            }
            using (RenameForm tbf = new RenameForm()
            {
                Owner = this
            })
            {               
                int si = lbComponents.SelectedIndex;
                if (tbf.ShowDialog() == DialogResult.OK)
                {                   
                    Components();
                    lbComponents.SelectedIndex = si;
                }
            }
        }
        #endregion Rename
        #region Arrange
        private void btArrange_Click(object sender, EventArgs e)
        {
            using (ArrangeComponentsForm arrcf = new ArrangeComponentsForm
            {
                Owner = this,
                dst = docFST._Header._Units._dist
            })
            {
                
                if (arrcf.ShowDialog() == DialogResult.OK)
                {

                }
            }
        }
        #endregion Arrange       
        #endregion Components
        #region LocalLibrary
        /// <summary>
        /// Функция заполнения ListView наименованиями библиотек
        /// </summary>
        private void LocalLibrary()
        {
            lbLibrary.Items.Clear();
            if (docFST._LocalLibrary._Packages?.Count > 0)
            {
                lbLibrary.Items.AddRange(docFST._LocalLibrary._Packages.ToArray());
            }
            Footprints();
        }
        #region AddToLibrary
        private void btAddLib_Click(object sender, EventArgs e)
        {
            using (AddLibraryForm al = new AddLibraryForm
            {
                Owner = this
            })
            {
               
                al.lbLibrary.Items.AddRange(docFST?._LocalLibrary?._Packages?.ToArray());
                if (al.lbLibrary.Items.Count > 0)
                {
                    foreach (LocalLibrary.Package p in al.lbLibrary.Items)
                    {
                        p.Tag = 0; //признак, что это упаковка из основного файла
                    }
                    if (al.ShowDialog() == DialogResult.OK)
                    {
                        LocalLibrary();
                    }
                }
            }
        }
        #endregion AddToLibrary
        #endregion LocalLibrary
        #region Footprints
        private void Footprints()
        {
            lbFootprints.Items.Clear();

            if (docFST._LocalLibrary._Footprints?.Count > 0)
            {
                lbFootprints.Items.AddRange(docFST._LocalLibrary._Footprints.ToArray());
            }
        }
        #endregion Footprints

        #endregion ComponentsPage

        #endregion TabControl

        private void toolStripButton1_Click(object sender, EventArgs e)
        {

        }
    }
}
