﻿using System;
using System.IO;
using System.Text;
using System.Xml.Serialization;
using System.Threading;

namespace TopoRAdd
{
    public class EventXmlSerializer : EventArgs
    {
        public string Message { get; }

        public EventXmlSerializer(string EventMessage)
        {
            Message = EventMessage;
        }
    }

    public static class XMLFile
    {

        public static event EventHandler<EventXmlSerializer> EventXmlSer;
        private static void UnknownAttribute(object sender, XmlAttributeEventArgs e)
        {
            EventXmlSer?.Invoke(null, new EventXmlSerializer("Unknown Attribute! " + Environment.NewLine + e.Attr.Name + Environment.NewLine +
                "Line number: " + e.LineNumber + Environment.NewLine + "Line position: " + e.LinePosition));
        }
        private static void UnknownElement(object sender, XmlElementEventArgs e)
        {
            EventXmlSer?.Invoke(null, new EventXmlSerializer("Unknown Element! " + Environment.NewLine + e.Element.Name + Environment.NewLine +
                "Line number: " + e.LineNumber + Environment.NewLine + "Line position: " + e.LinePosition));
        }
        private static void UnknownNode(object sender, XmlNodeEventArgs e)
        {
            EventXmlSer?.Invoke(null, new EventXmlSerializer("Unknown Node! " + Environment.NewLine + e.Name + Environment.NewLine +
               "Line number: " + e.LineNumber + Environment.NewLine + "Line position: " + e.LinePosition));
        }
        private static void UnreferencedObject(object sender, UnreferencedObjectEventArgs e)
        {
            EventXmlSer?.Invoke(null, new EventXmlSerializer("Unreferenced Object!" + Environment.NewLine + e.UnreferencedObject));
        }
  
        public static T LoadFST<T>(string filename)
        {
            using (StreamReader reader = new StreamReader(filename))
            {
                try
                {
                    XmlSerializer ser = new XmlSerializer(typeof(T));
                    ser.UnknownAttribute += UnknownAttribute;
                    ser.UnknownElement += UnknownElement;
                    ser.UnknownNode += UnknownNode;
                    ser.UnreferencedObject += UnreferencedObject;
                    var result = ser.Deserialize(reader);
                    return (T)result;
                }
                catch (InvalidOperationException ex)
                {
                    EventXmlSer?.Invoke(null, new EventXmlSerializer(ex.InnerException.ToString()));
                    return default;
                }
            }
        }
        public static void SaveFST<T>(T data, string filename)
        {
            using (StreamWriter writer = new StreamWriter(filename, false, Encoding.UTF8))
            {
                try
                {
                    XmlSerializer ser = new XmlSerializer(typeof(T));
                    ser.UnknownAttribute += UnknownAttribute;
                    ser.UnknownElement += UnknownElement;
                    ser.UnknownNode += UnknownNode;
                    ser.UnreferencedObject += UnreferencedObject;
                    XmlSerializerNamespaces ns = new XmlSerializerNamespaces();
                    ns.Add("", "");
                    ser.Serialize(writer, data, ns);
                }
                catch (InvalidOperationException ex)
                {
                    EventXmlSer?.Invoke(null, new EventXmlSerializer(ex.InnerException.ToString()));
                }
            }
        }
    }
}
