﻿/* Мною, Константином aka KilkennyCat, 05 июля 2020 года создано сиё 
 * на основе "Описание формата TopoR PCB версия 1.2.0 Апрель 2017 г.".
 * k@kilkennycat.pro
 * http://kilkennycat.ru  http://kilkennycat.pro
 */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Xml.Serialization;

namespace TopoR_PCB_Classes
{
    /// <summary>
    /// Описание конструктива платы.
    /// </summary>
    public class Constructive
    {
        /// <summary>
        /// Описание контура платы и вырезов в плате.
        /// </summary>
        public class BoardOutline
        {
            public class Shape_Contour
            {

                /// <summary>
                /// Толщина линии.
                /// </summary>
                [XmlAttribute("lineWidth", DataType = "float")]
                public float _lineWidth;

                /// <summary>
                /// Незалитая фигура.
                /// </summary>
                [
                    XmlElement("ArcCCW", typeof(ArcCCW)),
                    XmlElement("ArcCW", typeof(ArcCW)),
                    XmlElement("ArcByAngle", typeof(ArcByAngle)),
                    XmlElement("ArcByMiddle", typeof(ArcByMiddle)),
                    XmlElement("Circle", typeof(Circle)),
                    XmlElement("Line", typeof(Line)),
                    XmlElement("Polyline", typeof(Polyline)),
                    XmlElement("Rect", typeof(Rect)),
                    XmlElement("Contour", typeof(Contour))
                ]
                public object _NonfilledFigure;

                /*************************************************************************
                 * Здесь находятся функции для работы с элементами класса Shape_Contour. *
                 * Они не являются частью формата TopoR PCB.                             *
                 * ***********************************************************************/
                public void Shift(float x, float y)
                {
                    (_NonfilledFigure as IBaseFigure)?.Shift(x, y);
                }
                public void UnitsConvert(dist in_units, dist out_units)
                {
                    _lineWidth = Ut.UnitsConvert(_lineWidth, in_units, out_units);
                    (_NonfilledFigure as IBaseFigure)?.UnitsConvert(in_units, out_units);
                }
                /*************************************************************************/
            }
            public class Shape_Voids
            {

                /// <summary>
                /// Толщина линии.
                /// </summary>
                [XmlAttribute("lineWidth", DataType = "float")]
                public float _lineWidth;

                /// <summary>
                /// Описание залитой фигуры.
                /// </summary>
                [
                    XmlElement("FilledCircle", typeof(FilledCircle)),
                    XmlElement("FilledRect", typeof(FilledRect)),
                    XmlElement("Polygon", typeof(Polygon)),
                    XmlElement("FilledContour", typeof(FilledContour))
                ]
                public object _FilledFigure;

                /**********************************************************************
                * Здесь находятся функции для работы с элементами класса Shape_Voids. *
                * Они не являются частью формата TopoR PCB.                           *
                * *********************************************************************/
                public void Shift(float x, float y)
                {
                    (_FilledFigure as IBaseFigure)?.Shift(x, y);
                }
                public void UnitsConvert(dist in_units, dist out_units)
                {
                    _lineWidth = Ut.UnitsConvert(_lineWidth, in_units, out_units);
                    (_FilledFigure as IBaseFigure)?.UnitsConvert(in_units, out_units);
                }
                /***********************************************************************/
            }

            /// <summary>
            /// Описание контура платы.
            /// </summary>
            [XmlArray("Contour")]
            [XmlArrayItem("Shape")]
            public List<Shape_Contour> _Contours;
            public bool ShouldSerialize_Contours()
            {
                return _Contours?.Count > 0;
            }
            /// <summary>
            /// Вырезы в плате.
            /// </summary>
            [XmlArray("Voids")]
            [XmlArrayItem("Shape")]
            public List<Shape_Voids> _Voids;
            public bool ShouldSerialize_Voids()
            {
                return _Voids?.Count > 0;
            }
        }

        /// <summary>
        /// Описание монтажного отверстия на плате.
        /// </summary>
        public class MntholeInstance
        {
            /// <summary>
            /// Задаёт угол в градусах c точностью до тысячных долей.
            /// </summary>
            [XmlAttribute("angle", DataType = "float")]
            public float _angle;

            /// <summary>
            /// Признак фиксации.
            /// </summary>
            [XmlAttribute("fixed")]
            public Bool _fixed;
            [XmlIgnore]
            public bool _fixedSpecified { get { return _fixed != Bool.off; } }

            /// <summary>
            /// Ссылка на стек контактных площадок.
            /// </summary>
            [XmlElement("PadstackRef")]
            public PadstackRef _PadstackRef;

            /// <summary>
            /// ссылка на цепь.
            /// </summary>
            [XmlElement("NetRef")]
            public NetRef _NetRef;

            /// <summary>
            /// Точка привязки объекта.
            /// </summary>
            [XmlElement("Org")]
            public Org _Org;

            public void Shift(float x, float y)
            {
                _Org?.Shift(x, y);
            }
            public void UnitsConvert(dist in_units, dist out_units)
            {
                _Org?.UnitsConvert(in_units, out_units);
            }
        }
        /// <summary>
        /// Описание запрета.
        /// </summary>
        public class Keepout_Сonstructive
        {
            /// <summary>
            /// Тип запрета.
            /// </summary>
            public class Role
            {
                // <summary>
                /// Тип запрета: запрет трассировки.
                /// </summary>
                public class Trace
                {
                    /// <summary>
                    /// Тип запрета трассировки.
                    /// </summary>
                    [XmlAttribute("role")]
                    public role _role;

                    /// <summary>
                    /// Ссылка на слои. См. также _LayersRefs
                    /// </summary>
                    /// <remarks>! При null необходимо смотреть _LayersRefs - там описан список ссылок типа LayerRef. </remarks>
                    [
                         XmlElement("AllLayers", typeof(AllLayers)),
                         XmlElement("AllLayersInner", typeof(AllLayersInner)),
                         XmlElement("AllLayersInnerSignal", typeof(AllLayersInnerSignal)),
                         XmlElement("AllLayersSignal", typeof(AllLayersSignal)),
                         XmlElement("AllLayersOuter", typeof(AllLayersOuter)),
                         XmlElement("LayerGroupRef", typeof(LayerGroupRef))
                    ]
                    public object _LayersRef;
                    /// <summary>
                    /// Ссылка на слои. См. также _LayersRef
                    /// </summary>
                    /// <remarks>! При null необходимо смотреть _LayersRef - там описаны ссылки остальных типов. </remarks>
                    [XmlElement("LayerRef")]
                    public List<LayerRef>  _LayersRefs;
                    public bool ShouldSerialize_LayersRefs()
                    {
                        return _LayersRefs?.Count > 0;
                    }
                }

                /// <summary>
                /// Тип запрета: запрет размещения.
                /// </summary>
                public class Place
                {
                    /// <summary>
                    /// Сторона объекта.
                    /// </summary>
                    [XmlAttribute("side")]
                    public side _side;
                }
                /// <summary>
                /// Тип запрета: запрет трассировки.
                /// </summary>
                [XmlElement("Trace", typeof(Trace))]
                public Trace _Trace;
                [XmlElement("Place", typeof(Place))]
                public Place _Place;
            }

            [XmlElement("Role", typeof(Role))]
            public Role _Role;

            /// <summary>
            /// Описание фигуры.
            /// </summary>
            [
                XmlElement("ArcCCW", typeof(ArcCCW)),
                XmlElement("ArcCW", typeof(ArcCW)),
                XmlElement("ArcByAngle", typeof(ArcByAngle)),
                XmlElement("ArcByMiddle", typeof(ArcByMiddle)),
                XmlElement("Line", typeof(Line)),
                XmlElement("Circle", typeof(Circle)),
                XmlElement("Rect", typeof(Rect)),
                XmlElement("FilledCircle", typeof(FilledCircle)),
                XmlElement("FilledRect", typeof(FilledRect)),
                XmlElement("Polygon", typeof(Polygon)),

                XmlElement("Contour", typeof(Contour)),
                XmlElement("FilledContour", typeof(FilledContour)),

                XmlElement("Polyline", typeof(Polyline))
            ]
            public object _FigureContPolyline;

            /********************************************************************************
             * Здесь находятся функции для работы с элементами класса Keepout_Сonstructive. *
             * Они не являются частью формата TopoR PCB.                                    *
             * ******************************************************************************/
            public void Shift(float x, float y)
            {
                (_FigureContPolyline as IBaseFigure)?.Shift(x, y);
            }
            public void UnitsConvert(dist in_units, dist out_units)
            {
                (_FigureContPolyline as IBaseFigure)?.UnitsConvert(in_units, out_units);
            }
            /********************************************************************************/
        }

        /// <summary>
        /// Версия раздела.
        /// </summary>
        [XmlAttribute("version")]
        public string _version;

        /// <summary>
        /// Контур платы и вырезы в плате.
        /// </summary>
        [XmlElement("BoardOutline")]
        public BoardOutline _BoardOutline;

        /// <summary>
        /// Монтажные отверстия на плате.
        /// </summary>
        [XmlArray("Mntholes"), DefaultValue(null)]
        [XmlArrayItem("MntholeInstance")]
        public List<MntholeInstance> _Mntholes;
        public bool ShouldSerialize_Mntholes()
        {
            return _Mntholes?.Count > 0;
        }
        /// <summary>
        /// Детали на механических слоях.
        /// </summary>
        [XmlArray("MechLayerObjects"), DefaultValue(null)]
        [XmlArrayItem("Detail")]
        public List<Detail> _MechLayerObjects;
        public bool ShouldSerialize_MechLayerObjects()
        {
            return _MechLayerObjects?.Count > 0;
        }
        /// <summary>
        /// Описание надписей.
        /// </summary>
        [XmlArray("Texts"), DefaultValue(null)]
        [XmlArrayItem("Text")]
        public List<Text> _Texts;
        public bool ShouldSerialize_Texts()
        {
            return _Texts?.Count > 0;
        }
        /// <summary>
        /// Описание запретов.
        /// </summary>
        [XmlArray("Keepouts"), DefaultValue(null)]
        [XmlArrayItem("Keepout")]
        public List<Keepout_Сonstructive> _Keepouts;
        public bool ShouldSerialize_Keepouts()
        {
            return _Keepouts?.Count > 0;
        }

        /************************************************************************
         * Здесь находятся функции для работы с элементами класса Сonstructive. *
         * Они не являются частью формата TopoR PCB.                            *
         * **********************************************************************/

        public void Shift(float x, float y)
        {
            if (_BoardOutline != null)
            {
                if (_BoardOutline?._Contours.Count > 0)
                    for (int i = 0; i < _BoardOutline._Contours.Count; i++)
                        _BoardOutline._Contours[i].Shift(x, y);
                if (_BoardOutline?._Voids.Count > 0)
                    for (int i = 0; i < _BoardOutline._Voids.Count; i++)
                        _BoardOutline._Voids[i].Shift(x, y);
            }

            if (_Mntholes?.Count > 0)
                for (int i = 0; i < _Mntholes.Count; i++)
                    _Mntholes[i].Shift(x, y);

            if (_MechLayerObjects?.Count > 0)
                for (int i = 0; i < _MechLayerObjects.Count; i++)
                    _MechLayerObjects[i].Shift(x, y);

            if (_Texts?.Count > 0)
                for (int i = 0; i < _Texts.Count; i++)
                    _Texts[i].Shift(x, y);

            if (_Keepouts?.Count > 0)
                for (int i = 0; i < _Keepouts.Count; i++)
                    _Keepouts[i].Shift(x, y);
        }

        public void UnitsConvert(dist in_units, dist out_units)
        {
            if (_BoardOutline?._Contours?.Count > 0)
                for (int i = 0; i < _BoardOutline._Contours.Count; i++)
                    _BoardOutline._Contours[i].UnitsConvert(in_units, out_units);
            if (_BoardOutline?._Voids?.Count > 0)
                for (int i = 0; i < _BoardOutline._Voids.Count; i++)
                    _BoardOutline._Voids[i].UnitsConvert(in_units, out_units);


            if (_Mntholes?.Count > 0)
                for (int i = 0; i < _Mntholes.Count; i++)
                    _Mntholes[i].UnitsConvert(in_units, out_units);

            if (_MechLayerObjects?.Count > 0)
                for (int i = 0; i < _MechLayerObjects.Count; i++)
                    _MechLayerObjects[i].UnitsConvert(in_units, out_units);

            if (_Texts?.Count > 0)
                for (int i = 0; i < _Texts.Count; i++)
                    _Texts[i].UnitsConvert(in_units, out_units);

            if (_Keepouts?.Count > 0)
                for (int i = 0; i < _Keepouts.Count; i++)
                    _Keepouts[i].UnitsConvert(in_units, out_units);
        }

      /*   public void Add(Constructive a, bool boardOutline, bool mntholeInstances, bool details, bool texts, bool keepouts)
        {
           int l;
            if (boardOutline)
            {
                if (a?._BoardOutline?._Contours?.Count > 0)
                {
                    if (_BoardOutline == null)
                        _BoardOutline = new BoardOutline();

                    if (_BoardOutline._Contours == null)
                        _BoardOutline._Contours = (BoardOutline.Shape_Contour[])a._BoardOutline._Contours.Clone();
                    else
                    {
                        l = _BoardOutline._Contours.Count;
                        Array.Resize(ref _BoardOutline._Contours, l + a._BoardOutline._Contours.Count);
                        a._BoardOutline._Contours.CopyTo(_BoardOutline._Contours, l);
                    }
                }

                if (a?._BoardOutline?._Voids?.Count > 0)
                {
                    if (_BoardOutline == null)
                        _BoardOutline = new BoardOutline();

                    if (_BoardOutline._Voids == null)
                        _BoardOutline._Voids = (BoardOutline.Shape_Voids[])a._BoardOutline._Voids.Clone();
                    else
                    {
                        l = _BoardOutline._Voids.Count;
                        Array.Resize(ref _BoardOutline._Voids, l + a._BoardOutline._Voids.Count);
                        a._BoardOutline._Voids.CopyTo(_BoardOutline._Voids, l);
                    }
                }
            }

            if (mntholeInstances)
            {
                if (a?._Mntholes?.Count > 0)
                {
                    if (_Mntholes == null)
                        _Mntholes = (MntholeInstance[])a._Mntholes.Clone();
                    else
                    {
                        l = _Mntholes.Count;
                        Array.Resize(ref _Mntholes, l + a._Mntholes.Count);
                        a._Mntholes.CopyTo(_Mntholes, l);
                    }
                }
            }

            if (details)
            {
                if (a?._MechLayerObjects?.Count > 0)
                {
                    if (_MechLayerObjects == null)
                        _MechLayerObjects = (Detail[])a._MechLayerObjects.Clone();
                    else
                    {
                        l = _MechLayerObjects.Count;
                        Array.Resize(ref _MechLayerObjects, l + a._MechLayerObjects.Count);
                        a._MechLayerObjects.CopyTo(_MechLayerObjects, l);
                    }
                }
            }

            if (texts)
            {
                if (a?._Texts?.Count > 0)
                {
                    if (_Texts == null)
                        _Texts = (Text[])a._Texts.Clone();
                    else
                    {
                        l = _Texts.Count;
                        Array.Resize(ref _Texts, l + a._Texts.Count);
                        a._Texts.CopyTo(_Texts, l);
                    }
                }
            }

            if (keepouts)
            {
                if (a?._Keepouts?.Count > 0)
                {
                    if (_Keepouts == null)
                        _Keepouts = (Keepout_Сonstructive[])a._Keepouts.Clone();
                    else
                    {
                        l = _Keepouts.Count;
                        Array.Resize(ref _Keepouts, l + a._Keepouts.Count);
                        a._Keepouts.CopyTo(_Keepouts, l);
                    }
                }
            }
        }*/
        /************************************************************************/

    }
}