﻿/* Мною, Константином aka KilkennyCat, 05 июля 2020 года создано сиё 
 * на основе "Описание формата TopoR PCB версия 1.2.0 Апрель 2017 г.".
 * k@kilkennycat.pro
 * http://kilkennycat.ru  http://kilkennycat.pro
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml.Serialization;

namespace TopoR_PCB_Classes
{
    /// <summary>
    /// Раздел «Библиотечные элементы». (Обязательный раздел)
    /// </summary>
    public class LocalLibrary
    {
        public interface IBaseUnitConverter
        {
            void UnitsConvert(dist in_units, dist out_units);
        }
        public abstract class BasePad : IBaseUnitConverter
        {
            /// <summary>
            /// Ссылка на слой или тип слоя. 
            /// </summary>
            /// <value>LayerTypeRef | LayerRef</value>
            [
                XmlElement("LayerTypeRef", typeof(LayerTypeRef)),
                XmlElement("LayerRef", typeof(LayerRef))
            ]
            public object _Reference;
            public abstract void UnitsConvert(dist in_units, dist out_units);
        }
        /// <summary>
        /// Описание круглой контактной площадки.
        /// </summary>
        public class PadCircle :BasePad
        {
            /// <summary>
            /// Диаметр окружности, круга, овала.
            /// </summary>
            [XmlAttribute("diameter", DataType = "float")]
            public float _diameter;

            public override void UnitsConvert(dist in_units, dist out_units)
            {
                _diameter = Ut.UnitsConvert(_diameter, in_units, out_units);
            }
        }

        /// <summary>
        /// Описание овальной контактной площадки.
        /// </summary>
        public class PadOval : BasePad
        {
            /// <summary>
            /// Диаметр окружности, круга, овала.
            /// </summary>
            [XmlAttribute("diameter", DataType = "float")]
            public float _diameter;

            /// <summary>
            /// Параметр овальной контактной площадки: вытягивание по осям x и y.
            /// </summary>
            [XmlElement("Stretch")]
            public Stretch _Stretch;

            /// <summary>
            /// Параметр контактной площадки: смещение точки привязки по осям x и y.
            /// </summary>
            [XmlElement("Shift")]
            public Shift _Shift;

            public override void UnitsConvert(dist in_units, dist out_units)
            {
                _diameter = Ut.UnitsConvert(_diameter, in_units, out_units);
                _Stretch?.UnitsConvert(in_units, out_units);
                _Shift?.UnitsConvert(in_units, out_units);
            }
        }

        /// <summary>
        /// Описание прямоугольной контактной площадки.
        /// Дополнительные атрибуты(handling и handlingValue) позволяют задавать тип и величину обработки углов.
        /// В качестве типа обработки допускается скругление или срез.
        /// Тип для всех углов должен быть одинаковым: нельзя скруглять один угол и срезать другой.
        /// Если флаг custom не установлен, обрабатываются все углы, иначе будут обработаны только углы, 
        /// соответствующие установленным флагам - cornerLB, cornerRB, cornerRT, cornerLT.
        /// Основные формы КП, которые данный формат позволяет описать:
        /// прямоугольные КП;
        /// прямоугольные КП со скруглёнными углами;
        /// прямоугольные КП со срезанными углами;
        /// Finger pads.
        /// </summary>
        public class PadRect : BasePad
        {
            /// <summary>
            /// Ширина прямоугольной контактной площадки.
            /// </summary>
            [XmlAttribute("width", DataType = "float")]
            public float _width;

            /// <summary>
            /// Высота прямоугольной контактной площадки.
            /// </summary>
            [XmlAttribute("height", DataType = "float")]
            public float _height;

            /// <summary>
            /// Тип обработки углов прямоугольной контактной площадки.
            /// </summary>
            [XmlAttribute("handling")]
            public type_handling _handling;
            [XmlIgnore]
            public bool _handlingSpecified { get { return _handling != type_handling.None; } }

            /// <summary>
            /// Величина обработки углов прямоугольной контактной площадки. Значение зависит от типа обработки. Для скругления это радиус. Для среза это высота среза.
            /// </summary>
            [XmlAttribute("handlingValue", DataType = "float")]
            public float _handlingValue;
            [XmlIgnore]
            public bool _handlingValueSpecified { get { return _handlingValue != 0; } }

            /// <summary>
            /// Флаг выборочной обработки углов прямоугольной контактной площадки. Если не установлен, то все углы обрабатываются одинаковым образом.
            /// </summary>
            [XmlAttribute("custom")]
            public Bool _custom;
            [XmlIgnore]
            public bool _customSpecified { get { return _custom != Bool.off; } }

            /// <summary>
            /// Флаг обработки левого нижнего угла прямоугольной контактной площадки.
            /// </summary>
            [XmlAttribute("cornerLB")]
            public Bool _cornerLB;
            [XmlIgnore]
            public bool _cornerLBSpecified { get { return _cornerLB != Bool.off; } }

            /// <summary>
            /// Флаг обработки правого нижнего угла прямоугольной контактной площадки.
            /// </summary>
            [XmlAttribute("cornerRB")]
            public Bool _cornerRB;
            [XmlIgnore]
            public bool _cornerRBSpecified { get { return _cornerRB != Bool.off; } }

            /// <summary>
            /// Флаг обработки правого нижнего угла прямоугольной контактной площадки.
            /// </summary>
            [XmlAttribute("cornerRT")]
            public Bool _cornerRT;
            [XmlIgnore]
            public bool _cornerRTSpecified { get { return _cornerRT != Bool.off; } }

            /// <summary>
            /// Флаг обработки левого верхнего угла прямоугольной контактной площадки.
            /// </summary>
            [XmlAttribute("cornerLT")]
            public Bool _cornerLT;
            [XmlIgnore]
            public bool _cornerLTSpecified { get { return _cornerLT != Bool.off; } }

            /// <summary>
            /// Параметр контактной площадки: смещение точки привязки по осям x и y.
            /// </summary>
            [XmlElement("Shift")]
            public Shift _Shift;

            public override void UnitsConvert(dist in_units, dist out_units)
            {
                _width = Ut.UnitsConvert(_width, in_units, out_units);
                _height = Ut.UnitsConvert(_height, in_units, out_units);
                _handlingValue = Ut.UnitsConvert(_handlingValue, in_units, out_units);
                _Shift?.UnitsConvert(in_units, out_units);
            }
        }

        /// <summary>
        /// Описание полигональной контактной площадки.
        /// </summary>
        public class PadPoly : BasePad
        {
            /// <summary>
            /// Массив координат точек, вершин.
            /// </summary>
            /// <remarks>! Минимум 3 элемента</remarks>
            [XmlElement("Dot")]
            public List<Dot> _Dots;
            public bool ShouldSerialize_Dots()
            {
                return _Dots?.Count > 0;
            }

            public override void UnitsConvert(dist in_units, dist out_units)
            {
                if (_Dots != null)
                {
                    foreach (Dot d in _Dots)
                    {
                        d.UnitsConvert(in_units, out_units);
                    }
                }
            }
        }

        /// <summary>
        /// Описание стека контактных площадок.
        /// </summary>
        public class Padstack
        {
            /// <summary>
            /// Имя объекта или ссылка на именованный объект.
            /// </summary>
            [XmlAttribute("name")]
            public string _name;

            /// <summary>
            /// Тип стека контактных площадок.
            /// </summary>
            [XmlAttribute("type")]
            public type_padstack _type;

            /// <summary>
            /// Диаметр отверстия.
            /// </summary>
            [XmlAttribute("holeDiameter", DataType = "float")]
            public float _holeDiameter;

            /// <summary>
            /// Параметр стека контактной площадки: металлизация отверстия.
            /// </summary>
            [XmlAttribute("metallized")]
            public Bool _metallized;
            [XmlIgnore]
            public bool _metallizedSpecified { get { return _metallized != Bool.off; } }

            /// <summary>
            /// Параметр стека контактной площадки: подключение к области металлизации (полигону).
            /// </summary>
            [XmlAttribute("connectToCopper")]
            public type_connectToCopper _connectToCopper;

            /// <summary>
            /// Описание термобарьера.
            /// </summary>
            [XmlElement("Thermal")]
            public Thermal _Thermal;

            /// <summary>
            /// Контактные площадки стека.
            /// </summary>
            /// <value>PadCircle, PadOval, PadRect, PadPoly</value>
            [XmlArray("Pads")]
            [
                XmlArrayItem("PadCircle", typeof(PadCircle)),
                XmlArrayItem("PadOval", typeof(PadOval)),
                XmlArrayItem("PadRect", typeof(PadRect)),
                XmlArrayItem("PadPoly", typeof(PadPoly))
            ]
            public List<object> _Pads;
            public bool ShouldSerialize_Pads()
            {
                return _Pads?.Count > 0;
            }

            public void UnitsConvert(dist in_units, dist out_units)
            {
                _holeDiameter = Ut.UnitsConvert(_holeDiameter, in_units, out_units);
                _Thermal?.UnitsConvert(in_units, out_units);
                if (_Pads != null)
                {
                    foreach (var p in _Pads)
                    {
                        (p as IBaseUnitConverter)?.UnitsConvert(in_units, out_units);
                    }
                }
            }
        }

        /// <summary>
        /// Описание типа (стека) переходного отверстия.
        /// </summary>
        public class Viastack : IBaseUnitConverter
        {
            /// <summary>
            /// Диапазон слоев.
            /// </summary>
            /// <value>AllLayers | [LayerRef]</value>
            public class LayerRange
            {
                /// <summary>
                /// AllLayers - yстанавливает область действия правила: все слои. См. также _LayerRefs
                /// </summary>
                /// <remarks>! При null необходимо смотреть _LayersRefs - там описан список ссылок типа LayerRef. </remarks>
                [XmlElement("AllLayers", typeof(AllLayers))]
                public AllLayers _AllLayers;

                /// <summary>
                /// Диапазон слоёв. См. также _AllLayers
                /// </summary>
                /// <remarks>! При null необходимо смотреть наличие _AllLayers. </remarks>
                [XmlElement("LayerRef", typeof(LayerRef))]
                public List<LayerRef> _LayerRefs;
                public bool ShouldSerialize_LayerRefs()
                {
                    return _LayerRefs?.Count > 0;
                }

            }

            /// <summary>
            /// Имя объекта или ссылка на именованный объект.
            /// </summary>
            [XmlAttribute("name")]
            public string _name;

            /// <summary>
            /// Диаметр отверстия.
            /// </summary>
            [XmlAttribute("holeDiameter", DataType = "float")]
            public float _holeDiameter;

            /// <summary>
            /// Параметр типа переходного отверстия: возможность установить переходное отверстие на контактной площадке.
            /// </summary>
            [XmlAttribute("viaOnPin")]
            public Bool _viaOnPin;
            [XmlIgnore]
            public bool _viaOnPinSpecified { get { return _viaOnPin != Bool.off; } }

            /// <summary>
            /// Диапазон слоев.
            /// </summary>
            /// <value>AllLayers | [LayerRef]</value>
            [XmlElement("LayerRange", typeof(LayerRange))]
            public LayerRange _LayerRange;            

            /// <summary>
            /// Описание площадок стека переходного отверстия.
            /// </summary>
            [XmlArray("ViaPads")]
            [XmlArrayItem("PadCircle", typeof(PadCircle))]
            public List<PadCircle> _ViaPads;
            public bool ShouldSerialize_ViaPads()
            {
                return _ViaPads?.Count > 0;
            }

            public void UnitsConvert(dist in_units, dist out_units)
            {
                _holeDiameter = Ut.UnitsConvert(_holeDiameter, in_units, out_units);
                if (_ViaPads != null)
                {
                    foreach (PadCircle p in _ViaPads)
                    {
                        p.UnitsConvert(in_units, out_units);
                    }
                }
            }
        }

        /// <summary>
        /// Описание посадочного места.
        /// </summary>
        public class Footprint : IBaseUnitConverter
        {
            /// <summary>
            /// Описание области металлизации (полигона) в посадочном месте компонента.
            /// </summary>
            public class Copper_Footprint :IBaseUnitConverter
            {
                /// <summary>
                /// Толщина линии.
                /// </summary>
                [XmlAttribute("lineWidth", DataType = "float")]
                public float _lineWidth;

                /// <summary>
                /// Ссылка на слой.
                /// </summary>
                [XmlElement("LayerRef")]
                public LayerRef _LayerRef;

                /// <summary>
                /// Описание фигуры.
                /// </summary>
                /// <value>ArcCCW, ArcCW, ArcByAngle, ArcByMiddle, Line, Circle, Rect, FilledCircle, FilledRect, Polygon</value>
                [
                    XmlElement("ArcCCW", typeof(ArcCCW)),
                    XmlElement("ArcCW", typeof(ArcCW)),
                    XmlElement("ArcByAngle", typeof(ArcByAngle)),
                    XmlElement("ArcByMiddle", typeof(ArcByMiddle)),
                    XmlElement("Line", typeof(Line)),
                    XmlElement("Circle", typeof(Circle)),
                    XmlElement("Rect", typeof(Rect)),
                    XmlElement("FilledCircle", typeof(FilledCircle)),
                    XmlElement("FilledRect", typeof(FilledRect)),
                    XmlElement("Polygon", typeof(Polygon)),
                    XmlElement("FilledContour", typeof(FilledContour))
                ]
                public object _Figure;

                public void UnitsConvert(dist in_units, dist out_units)
                {
                    _lineWidth = Ut.UnitsConvert(_lineWidth, in_units, out_units);
                    (_Figure as IBaseFigure)?.UnitsConvert(in_units, out_units);
                }
            }

            /// <summary>
            /// Описание запрета в посадочном месте Footprint. Для запрета размещения должен быть указан слой с типом Assy.
            /// </summary>
            public class Keepout_Place_Trace : IBaseUnitConverter
            {
                /// <summary>
                /// Ссылка на слой.
                /// </summary>
                [XmlElement("LayerRef")]
                public LayerRef _LayerRef;

                /// <summary>
                /// Описание фигуры.
                /// </summary>
                /// <value>ArcCCW, ArcCW, ArcByAngle, ArcByMiddle, Line, Circle, Rect, FilledCircle, FilledRect, Polygon</value>
                [ 
                    XmlElement("ArcCCW", typeof(ArcCCW)),
                    XmlElement("ArcCW", typeof(ArcCW)),
                    XmlElement("ArcByAngle", typeof(ArcByAngle)),
                    XmlElement("ArcByMiddle", typeof(ArcByMiddle)),
                    XmlElement("Line", typeof(Line)),
                    XmlElement("Circle", typeof(Circle)),
                    XmlElement("Rect", typeof(Rect)),
                    XmlElement("FilledCircle", typeof(FilledCircle)),
                    XmlElement("FilledRect", typeof(FilledRect)),
                    XmlElement("Polygon", typeof(Polygon)),
                    XmlElement("FilledContour", typeof(FilledContour))
                ]
                public object _Figure;

                public void UnitsConvert(dist in_units, dist out_units)
                {
                    (_Figure as IBaseFigure)?.UnitsConvert(in_units, out_units);
                }
            }

            /// <summary>
            /// Описание монтажного отверстия в посадочном месте.
            /// </summary>
            public class Mnthole : IBaseUnitConverter
            {
                /// <summary>
                /// Идентификатор неименованных объектов.
                /// </summary>
                [XmlAttribute("id")]
                public string _id;

                /// <summary>
                /// Ссылка на стек контактных площадок.
                /// </summary>
                [XmlElement("PadstackRef")]
                public PadstackRef _PadstackRef;

                /// <summary>
                /// Точка привязки объекта.
                /// </summary>
                [XmlElement("Org")]
                public Org _Org;

                public void UnitsConvert(dist in_units, dist out_units)
                {
                    _Org?.UnitsConvert(in_units, out_units);
                }
            }

            /// <summary>
            /// Описание ярлыка в посадочном месте.
            /// </summary>
            public class Label_Footprint : IBaseUnitConverter
            {
                /// <summary>
                /// Имя объекта или ссылка на именованный объект.
                /// </summary>
                [XmlAttribute("name")]
                public string _name;

                /// <summary>
                /// Параметр надписей (ярлыков): способ выравнивания текста.
                /// </summary>
                [XmlAttribute("align")]
                public align _align;

                /// <summary>
                /// Задаёт угол в градусах c точностью до тысячных долей.
                /// </summary>
                [XmlAttribute("angle", DataType = "float")]
                public float _angle;

                /// <summary>
                /// Параметр надписей и ярлыков: зеркальность отображения.
                /// </summary>
                [XmlAttribute("mirror")]
                public Bool _mirror;
                [XmlIgnore]
                public bool _mirrorSpecified { get { return _mirror != Bool.off; } }

                /// <summary>
                /// Ссылка на слой.
                /// </summary>
                [XmlElement("LayerRef")]
                public LayerRef _LayerRef;

                /// <summary>
                /// Ссылка на стиль надписей.
                /// </summary>
                [XmlElement("TextStyleRef")]
                public TextStyleRef _TextStyleRef;

                /// <summary>
                /// Точка привязки объекта.
                /// </summary>
                [XmlElement("Org")]
                public Org _Org;

                public void UnitsConvert(dist in_units, dist out_units)
                {
                    _Org?.UnitsConvert(in_units, out_units);
                }
            }

            /// <summary>
            /// Описание контактной площадки (вывода) посадочного места.
            /// </summary>
            /// <remarks>! В системе TopoR поддерживаются планарные контакты на внешних металлических слоях и не поддерживаются на внутренних.
            /// Т.е.у планарного контакта может быть только одна площадка или на верхней стороне, или на нижней.
            /// В описании планарного контакта используется только слой Top.
            /// Это означает, что контактная площадка будет находиться на одной стороне с компонентом.
            /// Если же площадка находится на противоположной стороне, то должен быть установлен флаг flipped.
            /// Этот флаг устанавливается в описании контакта посадочного места.
            /// </remarks>
            public class Pad : IBaseUnitConverter
            {
                /// <summary>
                /// Номер контактной площадки (вывода) посадочного места.
                /// </summary>
                [XmlAttribute("padNum", DataType = "int")]
                public int _padNum;

                /// <summary>
                /// Имя объекта или ссылка на именованный объект.
                /// </summary>
                [XmlAttribute("name")]
                public string _name;

                /// <summary>
                /// Задаёт угол в градусах c точностью до тысячных долей.
                /// </summary>
                [XmlAttribute("angle", DataType = "float")]
                public float _angle;

                /// <summary>
                /// Параметр контакта (вывода) посадочного места: перевёрнутость. 
                /// Если флаг не установлен, площадка планарного контакта будет находиться на одной стороне с компонентом,
                /// иначе площадка будет расположена на противоположной стороне.
                /// </summary>
                [XmlAttribute("flipped")]
                public Bool _flipped;
                [XmlIgnore]
                public bool _flippedSpecified { get { return _flipped != Bool.off; } }

                /// <summary>
                /// Ссылка на стек контактных площадок.
                /// </summary>
                [XmlElement("PadstackRef")]
                public PadstackRef _PadstackRef;

                /// <summary>
                /// Точка привязки объекта.
                /// </summary>
                [XmlElement("Org")]
                public Org _Org;

                public void UnitsConvert(dist in_units, dist out_units)
                {
                    _Org?.UnitsConvert(in_units, out_units);
                }
            }

            /// <summary>
            /// Имя объекта или ссылка на именованный объект.
            /// </summary>
            [XmlAttribute("name")]
            public string _name;

            /// <summary>
            /// Описание контактных площадок посадочного места.
            /// </summary>
            [XmlArray("Pads")]
            [XmlArrayItem("Pad")]
            public List<Pad> _Pads;
            public bool ShouldSerialize_Pads()
            {
                return _Pads?.Count > 0;
            }
            /// <summary>
            /// Надписи.
            /// </summary>
            [XmlArray("Texts")]
            [XmlArrayItem("Text")]
            public List<Text> _Texts;
            public bool ShouldSerialize_Texts()
            {
                return _Texts?.Count > 0;
            }
            /// <summary>
            /// Детали посадочного места.
            /// </summary>
            [XmlArray("Details")]
            [XmlArrayItem("Detail")]
            public List<Detail> _Details;
            public bool ShouldSerialize_Details()
            {
                return _Details?.Count > 0;
            }
            /// <summary>
            /// Области металлизации (полигонов) в посадочных местах компонентов.
            /// </summary>
            [XmlArray("Coppers")]
            [XmlArrayItem("Copper")]
            public List<Copper_Footprint> _Coppers;
            public bool ShouldSerialize_Coppers()
            {
                return _Coppers?.Count > 0;
            }
            /// <summary>
            /// Запреты размещения в посадочном месте.
            /// </summary>
            [XmlArray("KeepoutsPlace")]
            [XmlArrayItem("Keepout")]
            public List<Keepout_Place_Trace> _KeepoutsPlace;
            public bool ShouldSerialize_KeepoutsPlace()
            {
                return _KeepoutsPlace?.Count > 0;
            }
            /// <summary>
            /// Запреты трассировки в посадочном месте.
            /// </summary>
            [XmlArray("KeepoutsTrace")]
            [XmlArrayItem("Keepout")]
            public List<Keepout_Place_Trace> _KeepoutsTrace;
            public bool ShouldSerialize_KeepoutsTrace()
            {
                return _KeepoutsTrace?.Count > 0;
            }
            /// <summary>
            /// Монтажные отверстия.
            /// </summary>
            [XmlArray("Mntholes")]
            [XmlArrayItem("Mnthole")]
            public List<Mnthole> _Mntholes;
            public bool ShouldSerialize_Mntholes()
            {
                return _Mntholes?.Count > 0;
            }
            /// <summary>
            /// Ярлыки.
            /// </summary>
            [XmlArray("Labels")]
            [XmlArrayItem("Label")]
            public List<Label_Footprint> _Labels;
            public bool ShouldSerialize_Labels()
            {
                return _Labels?.Count > 0;
            }

            public override string ToString()
            {
                return _name;
            }

            public void UnitsConvert(dist in_units, dist out_units)
            {
                _unitsconvert(_Pads, in_units, out_units);
                _unitsconvert(_Texts, in_units, out_units);
                _unitsconvert(_Details, in_units, out_units);
                _unitsconvert(_Coppers, in_units, out_units);
                _unitsconvert(_KeepoutsPlace, in_units, out_units);
                _unitsconvert(_KeepoutsTrace, in_units, out_units);
                _unitsconvert(_Mntholes, in_units, out_units);
                _unitsconvert(_Labels, in_units, out_units);
            }
        }


        /// <summary>
        /// Описание схемного компонента.
        /// </summary>
        public class Component
        {
            /// <summary>
            /// Описание контакта схемного компонента.
            /// </summary>
            public class Pin_Component
            {
                /// <summary>
                /// Номер контакта компонента.
                /// </summary>
                [XmlAttribute("pinNum", DataType = "int")]
                public int _pinNum;

                /// <summary>
                /// Имя объекта или ссылка на именованный объект.
                /// </summary>
                [XmlAttribute("name")]
                public string _name;

                /// <summary>
                /// Схемотехническое имя контакта компонента.
                /// </summary>
                [XmlAttribute("pinSymName")]
                public string _pinSymName;

                /// <summary>
                /// Параметр контакта компонента: эквивалентность.
                /// </summary>
                [XmlAttribute("pinEqual", DataType = "int")]
                public int _pinEqual;

                /// <summary>
                /// Параметр контакта (вывода) компонента: номер вентиля контакта.
                /// </summary>
                [XmlAttribute("gate", DataType = "int")]
                public int _gate;

                /// <summary>
                /// Параметр контакта (вывода) компонента: эквивалентность вентиля контакта.
                /// </summary>
                [XmlAttribute("gateEqual", DataType = "int")]
                public int _gateEqual;
            }

            /// <summary>
            /// Описание атрибута схемного компонента.
            /// </summary>
            public class Attribute_Component
            {
                /// <summary>
                /// Имя объекта или ссылка на именованный объект.
                /// </summary>
                [XmlAttribute("name")]
                public string _name;

                /// <summary>
                /// Значение атрибута.
                /// </summary>
                [XmlAttribute("value")]
                public string _value;
            }

            /// <summary>
            /// Имя объекта или ссылка на именованный объект.
            /// </summary>
            [XmlAttribute("name")]
            public string _name;

            /// <summary>
            /// Контакты схемного компонента.
            /// </summary>
            [XmlArray("Pins")]
            [XmlArrayItem("Pin")]
            public List<Pin_Component> _Pins;
            public bool ShouldSerialize_Pins()
            {
                return _Pins?.Count > 0;
            }
            /// <summary>
            /// Атрибуты компонента.
            /// </summary>
            [XmlArray("Attributes")]
            [XmlArrayItem("Attribute")]
            public List<Attribute_Component> _Attributes;
            public bool ShouldSerialize_Attributes()
            {
                return _Attributes?.Count > 0;
            }

            public override string ToString()
            {
                return _name;
            }
        }

        /// <summary>
        /// Описание упаковки (соответствие контактов компонента и выводов посадочного места).
        /// </summary>
        public class Package
        {
            /// <summary>
            /// Соответствие контакта схемного компонента и вывода посадочного места.
            /// </summary>
            public class Pinpack
            {
                /// <summary>
                /// Номер контакта компонента.
                /// </summary>
                [XmlAttribute("pinNum", DataType = "int")]
                public int _pinNum;

                /// <summary>
                /// Номер контактной площадки (вывода) посадочного места.
                /// </summary>
                [XmlAttribute("padNum", DataType = "int")]
                public int _padNum;

                /// <summary>
                /// Параметр правил выравнивания задержек: тип значений констант и допусков.
                /// </summary>
                [XmlAttribute("valueType")]
                public valueType _valueType;

                /// <summary>
                /// Параметр контакта компонента в посадочном месте: задержка сигнала в посадочном месте.
                /// </summary>
                [XmlAttribute("delay", DataType = "float")]
                public float _delay;
            }

            /// <summary>
            /// Ссылка на схемный компонент.
            /// </summary>
            [XmlElement("ComponentRef")]
            public ComponentRef _ComponentRef;

            /// <summary>
            /// Ссылка на посадочное место.
            /// </summary>
            [XmlElement("FootprintRef")]
            public FootprintRef _FootprintRef;

            /// <summary>
            /// Соответствие контакта схемного компонента и вывода посадочного места.
            /// </summary>
            [XmlElement("Pinpack")]
            public List<Pinpack> _Pinpacks;
            public bool ShouldSerialize_Pinpacks()
            {
                return _Pinpacks?.Count > 0;
            }

            /******************************************************************
            * Здесь находятся функции для работы с элементами класса Package. *
            * Они не являются частью формата TopoR PCB.                       *
            * *****************************************************************/
            public override string ToString()
            {
                return _ComponentRef._ReferenceName;
            }
            /**************************************************************/
            /// <summary>
            /// Универсальное поле для различных сервисных функций
            /// </summary>
            [XmlIgnore]
            public object Tag;

        }

        /// <summary>
        /// Версия раздела.
        /// </summary>
        [XmlAttribute("version")]
        public string _version;

        /// <summary>
        /// Стеки контактных площадок.
        /// </summary>
        [XmlArray("Padstacks")]
        [XmlArrayItem("Padstack")]
        public List<Padstack> _Padstacks;
        public bool ShouldSerialize_Padstacks()
        {
            return _Padstacks?.Count > 0;
        }
        /// <summary>
        /// Типы (стеки) переходных отверстий.
        /// </summary>
        [XmlArray("Viastacks")]
        [XmlArrayItem("Viastack")]
        public List<Viastack> _Viastacks;
        public bool ShouldSerialize_Viastacks()
        {
            return _Viastacks?.Count > 0;
        }
        /// <summary>
        /// Посадочные места.
        /// </summary>
        [XmlArray("Footprints")]
        [XmlArrayItem("Footprint")]
        public List<Footprint> _Footprints;
        public bool ShouldSerialize_Footprints()
        {
            return _Footprints?.Count > 0;
        }
        /// <summary>
        /// Схемные компоненты.
        /// </summary>
        [XmlArray("Components")]
        [XmlArrayItem("Component")]
        public List<Component> _Components;
        public bool ShouldSerialize_Components()
        {
            return _Components?.Count > 0;
        }
        /// <summary>
        /// Упаковки.
        /// </summary>
        [XmlArray("Packages")]
        [XmlArrayItem("Package")]
        public List<Package> _Packages;
        public bool ShouldSerialize_Packages()
        {
            return _Packages?.Count > 0;
        }


        /************************************************************************
         * Здесь находятся функции для работы с элементами класса LocalLibrary. *
         * Они не являются частью формата TopoR PCB.                            *
         * **********************************************************************/

        public void UnitsConvert(dist in_units, dist out_units)
        {
            _unitsconvert(_Padstacks, in_units, out_units);
            _unitsconvert(_Viastacks, in_units, out_units);
            _unitsconvert(_Footprints, in_units, out_units);
        }

        public bool AddPackage(LocalLibrary fromLibrary, dist in_units, dist out_units, Package package)
        {
            try
            {
                Package pack = Ut.DeepCopy(package);

                Component comp = Ut.DeepCopy(fromLibrary.ComponentByName(package._ComponentRef._ReferenceName));
                if (ComponentIndexOf(pack._ComponentRef._ReferenceName) != -1)
                {
                    pack._ComponentRef._ReferenceName += "_2";
                    while (ComponentIndexOf(pack._ComponentRef._ReferenceName) != -1)
                    {
                        pack._ComponentRef._ReferenceName = Ut.RefDesIncrement(pack._ComponentRef._ReferenceName, false);
                    }
                }
                comp._name = pack._ComponentRef._ReferenceName;
                _Components.Add(comp);

                Footprint foot = Ut.DeepCopy(fromLibrary.FootprintByName(package._FootprintRef._ReferenceName));
                if (in_units != out_units)
                {
                    foot.UnitsConvert(in_units, out_units);
                }                
                if (FootprintIndexOf(pack._FootprintRef._ReferenceName) != -1)
                {
                    pack._FootprintRef._ReferenceName += "_2";
                    while (FootprintIndexOf(pack._FootprintRef._ReferenceName) != -1)
                    {
                        pack._FootprintRef._ReferenceName = Ut.RefDesIncrement(pack._FootprintRef._ReferenceName, false);
                    }
                }                
                foot._name = pack._FootprintRef._ReferenceName;

                List<string> temppadstack = new List<string>();
                foreach (Footprint.Pad fp in foot._Pads)
                {
                    if (!temppadstack.Contains(fp._PadstackRef._ReferenceName))
                    {
                        temppadstack.Add(fp._PadstackRef._ReferenceName);
                    }
                }
                foreach (string namepadstack in temppadstack)
                {
                    Padstack padstack = Ut.DeepCopy(fromLibrary.PadstackByName(namepadstack));
                    if (PadstackIndexOf(padstack._name) != -1)
                    {
                        padstack._name += "_2";
                        while (PadstackIndexOf(padstack._name) != -1)
                        {
                            padstack._name = Ut.RefDesIncrement(padstack._name, false);
                        }
                    }
                    foreach (Footprint.Pad fp in foot._Pads.Where(pp => pp._PadstackRef._ReferenceName == namepadstack))
                    {
                        fp._PadstackRef._ReferenceName = padstack._name;
                    }
                    foreach (Detail dt in foot._Details)
                    {
                        dt._LayerRef = new LayerTypeRef()
                        { 
                            _type = layer_type.Silk 
                        };
                        dt.UnitsConvert(in_units, out_units);
                    }
                    if (in_units != out_units)
                    {
                        padstack.UnitsConvert(in_units, out_units);
                    }
                    _Padstacks.Add(padstack);
                }




                //TODO:и ещё деталям слой
                foot._KeepoutsPlace = null;
                foot._KeepoutsTrace = null;
                foot._Labels = null;
                foot._Mntholes = null;

                _Footprints.Add(foot);

                pack._ComponentRef._ReferenceName = foot._name;
                pack._FootprintRef._ReferenceName = comp._name;
                _Packages.Add(pack);
            }
            catch
            { }

            return true;
        }

        /// <summary>
        /// Индекс компонента по имени
        /// </summary>
        /// <param name="name">уникальное имя компонента</param>
        /// <returns>индекс компонента или -1, если компонент отсутствует</returns>
        public int ComponentIndexOf(string name)
        {
            for (int x = _Components?.Count ?? 0; x > 0; x--)
            {
                if (_Components[x - 1]._name == name)
                {
                    return x - 1;
                }
            }
            return -1;
        }

        public Component ComponentByName(string name)
        {
            for (int x = _Components?.Count ?? 0; x > 0; x--)
            {
                if (_Components[x - 1]._name == name)
                {
                    return _Components[x - 1];
                }
            }
            return null;
        }
        public int FootprintIndexOf(string name)
        {
            for (int x = _Footprints?.Count ?? 0; x > 0; x--)
            {
                if (_Footprints[x - 1]._name == name)
                {
                    return x - 1;
                }
            }
            return -1;
        }
        public Footprint FootprintByName(string name)
        {
            for (int x = _Footprints?.Count ?? 0; x > 0; x--)
            {
                if (_Footprints[x - 1]._name == name)
                {
                    return _Footprints[x - 1];
                }
            }
            return null;
        }

        public int PadstackIndexOf(string name)
        {
            for (int x = _Padstacks?.Count ?? 0; x > 0; x--)
            {
                if (_Padstacks[x - 1]._name == name)
                {
                    return x - 1;
                }
            }
            return -1;
        }

        public Padstack PadstackByName(string name)
        {
            for (int x = _Padstacks?.Count ?? 0; x > 0; x--)
            {
                if (_Padstacks[x - 1]._name == name)
                {
                    return _Padstacks[x - 1];
                }
            }
            return null;
        }

        private static void _unitsconvert<T>(List<T> obj, dist in_units, dist out_units)
        {
            if (obj != null)
            {
                foreach (var p in obj)
                {
                    (p as IBaseUnitConverter)?.UnitsConvert(in_units, out_units);
                }
            }
        }
        /************************************************************************/

    }
}
